#version 330 core

uniform sampler2D diffuse;
uniform sampler2DShadow depth;

uniform float emissive;

in vec2 uv;
in vec4 extinction;
in vec3 inscatter;
in vec3 eyeDir;
in vec3 lightDir;
in vec3 normal;
in vec4 shadow_coord;

layout (location = 0) out vec4 frag_color;

#include <atmosphere_model_include.vert>

#include <shading_models/oren_nayar_model_include.frag>

#include <forward/shadows_include.frag>

void main()
{
	vec3 ld = normalize(lightDir);
	vec3 ed = normalize(eyeDir);
	vec3 n = normalize(normal);

	vec3 terms = calculate_oren_nayar_model(n, ld, ed, 1.0, 8.0);

	vec4 diff = texture(diffuse, uv);
	vec4 color = diff*terms.x + diff*terms.y + diff*0.1;

	color *= soft_shadow(depth, shadow_coord, 16.0/1024.0)*0.75 + 0.25;

	lightGroundAtmosphere(color, extinction, inscatter);

	frag_color = color + diff*emissive + vec4(emissive*0.25);
}
